package com.tourism.common.huaweicloud.moderation.service;

import com.huaweicloud.sdk.core.auth.BasicCredentials;
import com.huaweicloud.sdk.core.auth.ICredential;
import com.huaweicloud.sdk.core.exception.ServiceResponseException;
import com.huaweicloud.sdk.moderation.v3.ModerationClient;
import com.huaweicloud.sdk.moderation.v3.model.*;
import com.huaweicloud.sdk.moderation.v3.region.ModerationRegion;
import com.tourism.common.huaweicloud.moderation.config.HuaweiCloudConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;

@Service
public class HuaweiCloudModerationService {

    @Autowired
    private HuaweiCloudConfig huaweiCloudConfig;

    public Boolean runTextModeration(String text) {
        ICredential credential = new BasicCredentials()
                .withProjectId(huaweiCloudConfig.getProjectId())
                .withAk(huaweiCloudConfig.getAccessKey())
                .withSk(huaweiCloudConfig.getSecretKey());

        ModerationClient client = ModerationClient.newBuilder()
                .withCredential(credential)
                .withRegion(ModerationRegion.valueOf(huaweiCloudConfig.getRegion()))
                .build();

        RunTextModerationRequest request = new RunTextModerationRequest();
        TextDetectionReq body = new TextDetectionReq();
        TextDetectionDataReq databody = new TextDetectionDataReq();
        databody.withText(text);
        body.withCategories(Arrays.asList("terrorism", "porn", "abuse", "ban"));
        body.withData(databody);
        body.withEventType("comment");
        request.withBody(body);
        try {
            RunTextModerationResponse runTextModerationResponse = client.runTextModeration(request);
            if("pass".equals(runTextModerationResponse.getResult().getSuggestion())){
                return true;
            }else {
                return false;
            }
        } catch (ServiceResponseException e) {
            // 处理异常
            e.printStackTrace();
            return false;
        }
    }

    public Boolean runImageModeration(String url) {
        ICredential credential = new BasicCredentials()
                .withProjectId(huaweiCloudConfig.getProjectId())
                .withAk(huaweiCloudConfig.getAccessKey())
                .withSk(huaweiCloudConfig.getSecretKey());

        ModerationClient client = ModerationClient.newBuilder()
                .withCredential(credential)
                .withRegion(ModerationRegion.valueOf(huaweiCloudConfig.getRegion()))
                .build();

        CheckImageModerationRequest request = new CheckImageModerationRequest();
        ImageDetectionReq body = new ImageDetectionReq();
        body.withCategories(Arrays.asList("terrorism", "porn", "abuse", "ban"));
        body.withUrl(url);
        body.withEventType("head_image");
        request.withBody(body);
        try {
            CheckImageModerationResponse response = client.checkImageModeration(request);
            if("pass".equals(response.getResult().getSuggestion())){
                return true;
            }else {
                return false;
            }
        } catch (ServiceResponseException e) {
            // 处理异常
            e.printStackTrace();
            return false;
        }
    }
}
